import scipy as sp

from pygradflow.params import LinearSolverType


class MINRESSolver:
    def __init__(self, mat):
        self.mat = mat

    def solve(self, b):
        return sp.sparse.linalg.minres(self.mat, b)[0]


class GMRESSolver:
    def __init__(self, mat):
        self.mat = mat

    def solve(self, b):
        return sp.sparse.linalg.gmres(self.mat, b, atol=0.)[0]


def linear_solver(mat, solver_type):
    if solver_type == LinearSolverType.LU:
        return sp.sparse.linalg.splu(mat)
    elif solver_type == LinearSolverType.MINRES:
        return MINRESSolver(mat)
    else:
        assert solver_type == LinearSolverType.GMRES

        return GMRESSolver(mat)
